/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Resolver = void 0;
const vscode_textmate_1 = require("vscode-textmate");
const fs = require("fs");
class Resolver {
    constructor(languages, onigLibPromise, onigLibName) {
        this.langMap = {};
        this.scopeToLangMap = {};
        this._languages = languages;
        this._onigLibPromise = onigLibPromise;
        this._onigLibName = onigLibName;
        this._languages.forEach(l => {
            this.langMap[l.id] = l;
            l.aliases.forEach(a => {
                this.langMap[a] = l;
            });
            this.scopeToLangMap[l.scopeName] = l;
        });
    }
    getOnigLib() {
        return this._onigLibPromise;
    }
    getOnigLibName() {
        return this._onigLibName;
    }
    getLangRegistration(langIdOrAlias) {
        return this.langMap[langIdOrAlias];
    }
    loadGrammar(scopeName) {
        return __awaiter(this, void 0, void 0, function* () {
            const lang = this.scopeToLangMap[scopeName];
            if (!lang) {
                return null;
            }
            if (lang.grammar) {
                return lang.grammar;
            }
            const g = yield readGrammarFromPath(lang.path);
            lang.grammar = g;
            return g;
        });
    }
}
exports.Resolver = Resolver;
function readGrammarFromPath(path) {
    return new Promise((c, e) => {
        fs.readFile(path, (error, content) => {
            if (error) {
                e(error);
            }
            else {
                c(vscode_textmate_1.parseRawGrammar(content.toString(), path));
            }
        });
    });
}
